<?xml version="1.0" ?>
<!-- Author: Brian Lynn, Global Electronic Markets LLC, for DSWG -->
<!-- Aug-Nov 2004 -->
<xsl:stylesheet 
	xmlns:xsl ="http://www.w3.org/1999/XSL/Transform" version="1.0"
	xmlns:fpml= "http://www.fpml.org/2004/FpML-4-1"
	xmlns:dswg= "http://www.datastandardsworkinggroup.org/position-reporting"
	xmlns:common="urn:schemas-microsoft-com:xslt"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
	>

	<!--
	For Microsoft:
	xmlns:common="urn:schemas-microsoft-com:xslt"
	For EXSLT processors (most non-MS parsers):
	xmlns:common="http://exslt.org/common"
	-->

<xsl:param name="filename"/>	<!-- filename being validated - for output -->
<xsl:param name="preferred.ccy" select="'USD'" />	<!-- currency preferred for reporting -->

<xsl:output method="text" encoding="iso-8859-1"/>	<!-- output is in text format -->

<!-- handling of coding scheme validation/floating rate indexes -->
<xsl:variable name="coding.schemes" select="document('coding_schemes.xml')"/>
<xsl:variable name="float.indexes" select="$coding.schemes/codingSchemes/codingScheme[@name='floatingRateIndex']"/>
<xsl:variable name="float.idx.ccysTF">
	<xsl:for-each select="$float.indexes/val">
		<val><xsl:value-of select="substring(.,1,3)"/></val>
	</xsl:for-each>
</xsl:variable>
<xsl:variable name="float.idx.ccys" select="common:node-set($float.idx.ccysTF)"/>


<!-- root template -->
<xsl:template match="/">

<!-- report the file name -->
<xsl:text>File: </xsl:text><xsl:value-of select="$filename"/><xsl:text>
</xsl:text>

<!-- validate data in positions -->
	<xsl:apply-templates mode="validate-data" select="fpml:FpML" />
	<xsl:apply-templates mode="validate-collateral" select="fpml:FpML/dswg:aggregateCollateral" />
	<xsl:apply-templates mode="stats" select="fpml:FpML"/>
<xsl:text>
</xsl:text>	<!-- newline in output -->
</xsl:template>


<!-- *** Data Validation *** -->
<xsl:template match="fpml:FpML[@dswgVersion='1-0' or @dswgVersion='1-1']" mode="validate-data">
	<xsl:if test="not(dswg:*)">DSWG.000 ERROR - Didn't find any elements in DSWG namespace</xsl:if>
	<xsl:apply-templates mode="validate-data" select="*" />
</xsl:template>

<xsl:template match="fpml:FpML[@dswgVersion='0-6']" mode="validate-data">
	<xsl:text>DSWG.003 WARNING - Validating DSWG version 0.6 (latest this validator supports is 1.0).
</xsl:text>
	<xsl:if test="not(dswg:*)">DSWG.000 ERROR - Didn't find any elements in DSWG namespace</xsl:if>
	<xsl:apply-templates mode="validate-data" select="*" />
</xsl:template>

<xsl:template match="fpml:FpML" mode="validate-data">DSWG.000 ERROR - Invalid DSWG Version (got <xsl:value-of select="@dswgVersion"/>, expected 1-0)</xsl:template>


<!-- do nothing by default - skip irrelevant nodes -->
<xsl:template match="node()" mode="validate-data" />


<!-- validate top level quote charateristics -->
<xsl:template match="dswg:quotationCharacteristics" mode="validate-data" >
	<xsl:if test="not(fpml:currency)">
(DSWG.002) ERROR - Reporting currency is not specified
	</xsl:if>
	<xsl:apply-templates select="." mode="validate-ccy" />
</xsl:template>

<xsl:template match="dswg:quotationCharacteristics" mode="validate-ccy" >
	<xsl:if test="not(fpml:currency)">
(DSWG.002) WARNING:  Reporting currency not specified.
	</xsl:if>
	<xsl:if test="fpml:currency and not(fpml:currency = $preferred.ccy)">
(DSWG.002) WARNING:  Reporting currency <xsl:value-of select="fpml:currency"/> is not preferred currency, <xsl:value-of select="$preferred.ccy" />.
	</xsl:if>
</xsl:template>

<!-- validate parties -->
<xsl:template match="dswg:party" mode="validate-data" >
	<xsl:if test="not(fpml:partyName)">
(DSWG.001) ERROR - Missing party name in party #<xsl:number />
	</xsl:if>
</xsl:template>


<!-- Validate data in position summaries or trade details -->

<xsl:template match="dswg:position" mode="validate-data" >
================================================
Doing position #<xsl:number />.

<xsl:text>
</xsl:text>
	<xsl:apply-templates mode="validate-ccy" select="dswg:quotationCharacteristics" />
	<xsl:apply-templates mode="do-position" select="dswg:positionSummary|dswg:trade" />
	<xsl:apply-templates mode="validate-collateral" select="dswg:positionSummary" />
</xsl:template>


<!-- positions and trades have an FpML product in second spot -->
<xsl:template match="dswg:positionSummary|dswg:trade" mode="do-position" >
	<xsl:apply-templates mode="validate-missing" select="." />
	<xsl:apply-templates mode="validate-data" select="dswg:positionHeader" />
	<xsl:apply-templates mode="do-product" select="*[2]" />
	<!-- validate that valuations are present where required and properly filled in -->
	<xsl:apply-templates mode="val-valuations" select="." />
	<xsl:apply-templates mode="val-calculations" select=".//dswg:valuations" />
	<xsl:apply-templates mode="val-quotations" select=".//dswg:valuations" />
	<!-- validate that calculated dates are present where required -->
	<xsl:apply-templates mode="val-scheduled-dates" select="." />
	<xsl:apply-templates mode="val-calculations" select=".//dswg:scheduledDates" />
</xsl:template>

<!-- validate receiver position ID -->
<xsl:template match="dswg:positionHeader" mode="validate-data" >
	<xsl:if test="not(dswg:receiverPositionIdentifier/text())">
(DSWG.002) WARNING: receiver position ID not supplied
</xsl:if>
<xsl:if test="not(dswg:tradeDate/text())">
(DSWG.004) ERROR: trade date not supplied
</xsl:if>
</xsl:template>


<!-- validate collateral info -->

<!-- not required for FX products -->

<!-- collateral must appear at individual trade level or at aggregate level, not both -->
<xsl:template match="dswg:positionSummary[fpml:fxSingleLeg|fpml:fxSimpleOption|fpml:fxDigitalOption|fpml:fxBarrierOption|fpml:fxAverageRateOption]" mode="validate-collateral" >
	
	<xsl:if test="dswg:collateral">
		<xsl:apply-templates mode="validate-indep" select="."/>
	</xsl:if>
	<!-- if the collateral info is not there, there must be a collateral reference -->
</xsl:template>

<xsl:template match="dswg:positionSummary" mode="validate-collateral" >
	<xsl:apply-templates mode="validate-indep" select="."/>
</xsl:template>

<xsl:template match="dswg:aggregateCollateral" mode="validate-collateral" >
<xsl:text>======================================================
Validating aggregate collateral for </xsl:text><xsl:value-of select="dswg:currency"/>
	<xsl:text>
</xsl:text>
	<xsl:apply-templates mode="validate-indep" select=".">
		<xsl:with-param name="pct" select="false"/>
	</xsl:apply-templates>
	<xsl:text>
</xsl:text>
</xsl:template>

<xsl:template match="node()" mode="validate-indep" >
	<xsl:param name="pct" select="true"/>
	<xsl:variable name="indep" select="dswg:collateral/fpml:independentAmount"/>
	<xsl:choose>
		<xsl:when test="not($indep)" >
(DSWG.030) WARNING: no collateral information supplied.
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="not($indep/fpml:paymentDetail/fpml:paymentAmount/fpml:amount)">
(DSWG.030a) WARNING: no independent amount supplied.</xsl:if>
			<xsl:if test="not($indep/fpml:paymentDetail/fpml:paymentAmount/fpml:currency/text())">
(DSWG.030b) WARNING: no independent amount currency supplied.</xsl:if>
			<xsl:if test="$pct and not($indep/fpml:paymentDetail/fpml:paymentRule/fpml:paymentPercent)">
(DSWG.030c) WARNING: no independent percentage supplied.</xsl:if>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:apply-templates mode="validate-currency-precision" select="dswg:collateral//fpml:amount"/>
</xsl:template>

<xsl:template match="dswg:collateralParty" mode="validate-collateral" >
	<xsl:if test="not(dswg:collateralParty/dswg:independentAmount | dswg:collateralParty/dswg:independentPercentage)" >
(DSWG.030d) WARNING: no independent amount/percentage supplied.
	</xsl:if>
</xsl:template>



<!-- ************   Product-related rules  ************ -->

<!-- display product - for progress tracking purposes -->
<xsl:template match="node()" mode="do-product" >(INFO) Product is: <xsl:value-of select="local-name(.)" />, type = <xsl:apply-templates select="." mode="productType"/><!--<xsl:apply-templates select="." mode="current-notional" />--><xsl:text>
</xsl:text>
	<xsl:apply-templates select="." mode="validate-productType" />
	<xsl:apply-templates mode="val-cross-product" select="." />
	<xsl:apply-templates mode="val-product" select="." />
</xsl:template>

<!--  (Current notional calculation removed in 0.6.2 
<xsl:template mode="current-notional" match="node()" />
<xsl:template mode="current-notional" match="fpml:swap" >,  notional = <xsl:apply-templates select="fpml:swapStream[1]" mode="current-notional"/></xsl:template>

<xsl:template mode="current-notional" match="fpml:swapStream" >
	<xsl:variable name="asOf" select="translate(/fpml:FpML/dswg:asOfDate,'-','')"/>
	<xsl:variable name="notlSched" select="fpml:calculationPeriodAmount/fpml:calculation/fpml:notionalSchedule/fpml:notionalStepSchedule"/>
	<xsl:variable name="steps" select="$notlSched/fpml:step[translate(fpml:stepDate,'-','') &lt;= $asOf]" />
	<xsl:value-of select="$notlSched [not(fpml:step[translate(fpml:stepDate,'-','') &lt;= $asOf])]/fpml:initialValue |  ($notlSched/fpml:step[translate(fpml:stepDate,'-','') &lt;= $asOf][last()])/fpml:stepValue" />
</xsl:template>
-->

<!-- **** validate across all products (generic rules) **** -->
<xsl:template match="node()" mode="val-cross-product" >
	<xsl:apply-templates select=".//fpml:businessCentersReference" mode="val-business-centers" />
</xsl:template>

<!-- **** validate specific product **** -->

<!-- Default product validation - do nothing -->
<xsl:template match="node()" mode="val-product" />

<!--	 (for debugging where required)
<xsl:template match="node()" mode="val-product">Unsupported product <xsl:value-of select="local-name(.)" />
</xsl:template>
-->


<!-- ******* validate ir swaps ******* -->
<xsl:template match="fpml:swap" mode="val-product" >
	<xsl:apply-templates mode="val-product" select="*" />
</xsl:template>

<xsl:template match="fpml:swapStream" mode="val-product" >
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:stubStartDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:stubEndDate/*" />
	<xsl:apply-templates mode="val-fixfloat" select="." />
	<xsl:apply-templates mode="val-irstream" select="." />
	<xsl:apply-templates mode="validate-currency-schedule" select=".//fpml:notionalStepSchedule"/>
	<xsl:apply-templates mode="validate-rate-schedule" select=".//fpml:fixedRateSchedule"/>
	<xsl:apply-templates mode="validate-rate-schedule" select=".//fpml:spreadSchedule"/>
	<xsl:apply-templates mode="validate-rate" select=".//fpml:stubRate"/>
</xsl:template>

<xsl:template match="fpml:swapStream" mode="val-irstream" >
	<xsl:apply-templates select="fpml:paymentDates/fpml:paymentDatesAdjustments" mode="require-business-centers" />
	<xsl:apply-templates select="fpml:paymentDates/fpml:paymentDatesAdjustments/fpml:businessDayConvention" mode="require-data" />
	<xsl:apply-templates select="fpml:calculationPeriodDates/fpml:calculationPeriodFrequency/fpml:rollConvention" mode="require-data" />
	<xsl:apply-templates select="fpml:calculationPeriodAmount/fpml:calculation/fpml:notionalSchedule/fpml:notionalStepSchedule/fpml:currency" mode="require-data" />
</xsl:template>

<!-- validate floating IRS streams -->
<xsl:template match="fpml:swapStream[fpml:calculationPeriodAmount/fpml:calculation/fpml:floatingRateCalculation]" mode="val-fixfloat" >
	<xsl:if test="not(fpml:resetDates)">
(DSWG.300) ERROR - Reset Dates must be supplied for floating streams.
	</xsl:if>
	<xsl:apply-templates select="fpml:resetDates/fpml:resetDatesAdjustments" mode="require-business-centers" />
	<xsl:if test="not(fpml:calculationPeriodAmount/fpml:calculation/fpml:floatingRateCalculation/fpml:indexTenor)">
(DSWG.301) WARNING: Index Tenor must normally be supplied for floating streams.  Unless this is an OIS, this is probably an error.
	</xsl:if>
	<xsl:apply-templates select="fpml:calculationPeriodAmount/fpml:calculation/fpml:dayCountFraction" mode="require-data" />
	<xsl:apply-templates select="fpml:calculationPeriodAmount/fpml:calculation/fpml:floatingRateCalculation/fpml:floatingRateIndex" mode="require-data" />
	<xsl:apply-templates select="fpml:calculationPeriodAmount/fpml:calculation/fpml:floatingRateCalculation/fpml:floatingRateIndex" mode="validate-index" />

	<xsl:if test="not(fpml:calculationPeriodAmount/fpml:calculation/fpml:floatingRateCalculation/fpml:spreadSchedule)">
(DSWG.302) ERROR - Spread schedule must be supplied for IRS floating streams, even if spread is 0.
	</xsl:if>
</xsl:template>


<!-- validate fixed IRS streams -->
<xsl:template match="fpml:swapStream" mode="val-fixfloat" >
	<xsl:apply-templates select="fpml:calculationPeriodAmount/fpml:calculation/fpml:dayCountFraction" mode="require-data" />
</xsl:template>

<!-- ******* validate ir options ******* -->
<xsl:template match="fpml:swaption | fpml:earlyTerminationProvision | fpml:cancelableProvision | fpml:extendibleProvision" mode="val-product" >
	<xsl:apply-templates mode="val-exercise" select=".//fpml:americanExercise | .//fpml:bermudaExercise | .//fpml:europeanExercise" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:cashSettlementPaymentDate/*" />
	<!-- validate underlying swap of swaption -->
	<xsl:apply-templates mode="val-product" select="fpml:swap" />

	<xsl:if test="local-name(.)='swaption' and not (fpml:premium)">
DSWG.350 ERROR - IR swaption premium is missing at <xsl:apply-templates select="." mode="location" /></xsl:if>
	<xsl:apply-templates mode="validate-currency-precision" select="fpml:premium//fpml:amount"/>
</xsl:template>

<xsl:template match="node()" mode="val-exercise" >
	<!-- currently undefined -->
	<!-- DSWG.101 - validate that only adjustable dates are provided, not relative dates -->
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:commencementDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:expirationDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:relevantUnderlyingDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:bermudaExerciseDates/*" />
</xsl:template>


<!-- ****** validate equity swaps ****** -->

<xsl:template match="fpml:equitySwap" mode="val-product" >

	<!-- DSWG.101 - validate that only adjustable dates are provided, not relative dates -->
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:effectiveDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:terminationDate/*" />
	<xsl:apply-templates mode="val-adj-date" select="fpml:equityLeg/fpml:valuation/fpml:equityPaymentDates/fpml:equityPaymentDatesInterim/*" />
	<xsl:apply-templates mode="val-adj-date" select="fpml:equityLeg/fpml:valuation/fpml:equityPaymentDates/fpml:equityPaymentDateFinal/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:stubStartDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:stubEndDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:startingDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:valuationDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:valuationDates/*" />
	<!-- we don't care about dividendPaymentDate  -->
	<!-- we don't care about calculationDates  -->

	<!-- validate underlyers -->
	<xsl:apply-templates mode="val-underlyers" select=".//fpml:underlyer" />

	<!-- validate dividend amount -->
	<xsl:apply-templates mode="val-dividend-amount" select="fpml:equityLeg/fpml:return" />
	<xsl:variable name="val" select="fpml:equityLeg/fpml:valuation"/>
	<xsl:apply-templates mode="val-commission" select="$val/fpml:initialPrice|$val/fpml:valuationPriceInterim|$val/fpml:valuationPriceFinal"/>

	<!-- validate individual fields in TRS -->
	<!-- DSWG.102 - validate that only explicit notionals are provided, not calculated ones -->
	<xsl:apply-templates mode="val-equity-notional" select=".//fpml:notional/*" />
	<xsl:apply-templates mode="val-equity-principal-exchange-amount" select=".//fpml:principalExchangeAmount/*" />

	<!-- field validations -->
	<xsl:if test="not(fpml:equityLeg/fpml:valuation/fpml:valuationPriceFinal/fpml:equityValuation/fpml:valuationDate/fpml:adjustableDate/fpml:unadjustedDate)">
DSWG.220 ERROR - TRS final valuation date missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not(fpml:equityLeg/fpml:equityAmount/fpml:paymentCurrency/fpml:currency)">
DSWG.222 ERROR - TRS settlement currency missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not(fpml:interestLeg/fpml:notional/fpml:notionalAmount/fpml:amount)">
DSWG.240 ERROR - TRS floating rate calculation amount missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:notional/fpml:notionalAmount/fpml:amount"/>
	<xsl:if test="not(fpml:interestLeg/fpml:interestCalculation/fpml:floatingRateCalculation/fpml:floatingRateIndex/text())">
DSWG.241 ERROR - TRS floating rate index missing at <xsl:apply-templates select="." mode="location" /></xsl:if>
	<xsl:apply-templates select="fpml:interestLeg/fpml:interestCalculation/fpml:floatingRateCalculation/fpml:floatingRateIndex" mode="validate-index"/>

	<xsl:apply-templates mode="validate-rate-schedule" select="fpml:interestLeg//fpml:fixedRateSchedule"/>
	<xsl:apply-templates mode="validate-rate-schedule" select="fpml:interestLeg//fpml:spreadSchedule"/>
	<xsl:apply-templates mode="validate-rate" select="fpml:interestLeg//fpml:stubRate"/>

	<xsl:if test="not(fpml:interestLeg/dswg:paymentFrequency)">
DSWG.242 ERROR - TRS floating payment frequency missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not(fpml:equityLeg/dswg:paymentFrequency)">
DSWG.243 ERROR - TRS total return payment frequency missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not(fpml:interestLeg/fpml:interestCalculation/fpml:floatingRateCalculation/fpml:indexTenor)">
DSWG.244 ERROR - TRS floating rate designated maturity missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not((fpml:equityLeg/fpml:valuation/fpml:equityPaymentDates/fpml:equityPaymentDatesInterim|fpml:equityLeg/fpml:valuation/fpml:equityPaymentDates/fpml:equityPaymentDateFinal)[1]//fpml:businessCenters)">
DSWG.245 ERROR - TRS payment business centers missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not(fpml:interestLeg/fpml:interestCalculation/fpml:dayCountFraction)">
DSWG.246 ERROR - TRS floating rate day count fraction missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not(fpml:interestLeg/fpml:interestCalculation/fpml:floatingRateCalculation/fpml:spreadSchedule)">
(DSWG.247) ERROR - Spread schedule must be supplied for IRS floating streams, even if spread is 0.
	</xsl:if>

</xsl:template>

<xsl:template match="fpml:equityOption[fpml:strike/@xsi:type='dswg:AssetSwapStrike']" mode="val-product" >
	<xsl:variable name="as" select="fpml:strike" />
	<xsl:apply-templates select="$as/dswg:resetDateAdjustments" mode="require-business-centers" />
	<xsl:apply-templates select="$as/dswg:paymentDateAdjustments" mode="require-business-centers" />
	<xsl:if test="not(dswg:makeWholeProvisions)">
DSWG.320 WARNING - Make whole provisions not present for asset swap option</xsl:if>
	<xsl:apply-templates mode="validate-rate" select="dswg:makeWholeProvisions/dswg:recallSpread"/>
	<xsl:apply-templates mode="validate-rate" select="$as/dswg:spread"/>
	<xsl:apply-templates select="." mode="val-eqo"/>
</xsl:template>

<xsl:template match="fpml:equityOption" mode="val-product" >
	<xsl:apply-templates select="." mode="val-eqo"/>
</xsl:template>

<xsl:template match="fpml:equityOption" mode="val-eqo" >
	<!-- validate underlyers -->
	<xsl:apply-templates mode="val-underlyers" select="fpml:underlyer" >
		<xsl:with-param name="dividend" select="0"/>
		<xsl:with-param name="full" select="0"/>
	</xsl:apply-templates>

	<xsl:if test="not(fpml:numberOfOptions)">
DSWG.601 ERROR - Number of options must be supplied for Equity Options.</xsl:if>

	<xsl:if test="not(fpml:strike/fpml:strikePrice)">
DSWG.602 ERROR - Strike price must be supplied for Equity Options.</xsl:if>

<!-- premium -->
	<xsl:if test="not(fpml:equityPremium/fpml:paymentAmount)">
DSWG.603 ERROR - Premium amount be supplied for Equity Options.</xsl:if>
	<xsl:if test="not(fpml:equityPremium/fpml:paymentDate)">
DSWG.604 ERROR - Premium payment date be supplied for Equity Options.</xsl:if>
	<xsl:apply-templates mode="require-business-centers" select="fpml:equityPremium/fpml:paymentDate"/>

<!-- exercise -->
	<xsl:if test="not(fpml:equityExercise/fpml:automaticExercise)">
DSWG.605 ERROR - Automatic Exercise indicator must be supplied for Equity Options.</xsl:if>

	<xsl:apply-templates mode="val-exercise" select="fpml:equityExercise/*"  />
	<xsl:apply-templates mode="require-business-centers" select="fpml:equityExercise//fpml:commencementDate|fpml:equityExercise//fpml:expirationDate|fpml:equityExercise//fpml:exerciseDate"/>

<!-- underlying -->
	<xsl:apply-templates mode="val-und-curr-price" select="fpml:underlyer/*"/>
</xsl:template>

<xsl:template mode="val-und-curr-price" match="fpml:basket">
	<xsl:apply-templates mode="val-und-curr-price" select="*"/>
</xsl:template>

<xsl:template mode="val-und-curr-price" match="node()" />

<xsl:template mode="val-und-curr-price" match="fpml:singleUnderlyer|fpml:basketConstituent">
	<xsl:if test="not(dswg:currentMarketPrice/fpml:netPrice)">
(DSWG.610) ERROR - Equity option underlyer net current market price is missing, at <xsl:apply-templates select="." mode="location"/>.</xsl:if>

	<xsl:if test="not(dswg:currentMarketPrice/fpml:netPrice/fpml:currency)">
(DSWG.611) ERROR - Equity option underlyer net current market price currency is missing, at <xsl:apply-templates select="." mode="location"/>.</xsl:if>

</xsl:template>
<!-- ****** validate variance swaps ****** -->

<xsl:template match="dswg:varianceSwap" mode="val-product" >
	<!-- TODO - rules for variance swaps -->
	<xsl:variable name="vleg" select="fpml:varianceLeg[1]"/>
	<xsl:variable name="var" select="$vleg/fpml:equityAmount/fpml:variance"/>

	<xsl:if test="not($vleg/dswg:effectiveDate)">
DSWG.260 ERROR - effective date not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:apply-templates mode="val-adj-date" select=".//fpml:valuationDate/*" />
	<xsl:apply-templates mode="val-adj-date" select=".//fpml:valuationDates/*" />

	<!-- validate underlyers -->
	<xsl:apply-templates mode="val-underlyers" select=".//fpml:underlyer" />

	<xsl:if test="not($var/dswg:priceType)">
DSWG.261 ERROR - cash/futures indicator not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not($var/dswg:varianceUnits)">
DSWG.262 ERROR - variance units not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not($var/fpml:volatilityStrikePrice)">
DSWG.263a ERROR - volatility strike price not specified for variance leg.</xsl:if>

	<xsl:if test="not($var/dswg:varianceStrikePrice)">
DSWG.263b ERROR - variance strike price not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not($var/dswg:dividendTreatment)">
DSWG.264 ERROR - dividend treatment not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not($var/dswg:annualizationFactor)">
DSWG.265 ERROR - annualization factor not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not($var/dswg:actualObservations)">
DSWG.266 ERROR - actual number of observations to date not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not(fpml:varianceLeg/fpml:equityAmount/fpml:paymentCurrency/fpml:currency)">
DSWG.267 ERROR - TRS settlement currency missing at <xsl:apply-templates select="." mode="location" /></xsl:if>

	<xsl:if test="not($var/fpml:expectedN)">
DSWG.268 ERROR - expected number of observations to date not specified for variance leg.</xsl:if>

	<xsl:if test="($var/dswg:varianceCap) and not($var/fpml:varianceCapPrice)">
DSWG.269 ERROR - variance cap price not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="($var/dswg:varianceCap) and not($var/fpml:volatilityCapPrice)">
DSWG.270 ERROR - volatility cap price not specified for variance leg (use DSWG extension type).</xsl:if>

	<xsl:if test="not($var/fpml:varianceCap)">
DSWG.271 WARNING - Variance Cap = false.</xsl:if>

	<xsl:apply-templates mode="validate-currency-precision" select="$var/fpml:varianceAmount/fpml:amount"/>

</xsl:template>


<!-- **** Beginning of individual rules **** --> 

<!-- valuation related rules -->

<!-- list of required quotes, by parent element name -->
<!-- corresponds to list of required valuations in DSWG spec -->
<xsl:variable name="required.quotations.tree.frag">
	<quote set="positionSummary" name="NPVLocalCurrency"  />
	<quote set="positionSummary" name="NPV"  />
	<quote set="swapStream" name="AccruedInterest" />
	<quote set="swapStream" name="CurrentNotional" />
	<quote set="creditDefaultSwap" name="AccruedInterest" />
	<quote set="creditDefaultSwap" name="RecoveryRate" precision="8" percentage="true"/>
	<quote set="creditDefaultSwap" name="CreditSpread" precision="8"/>
	<quote set="creditDefaultSwapBasket" name="AccruedInterest" />
	<quote set="referenceInformation" name="RecoveryRate" precision="8" percentage="true"/>
	<quote set="referenceInformation" name="CreditSpread" precision="8"/>
	<quote set="interestLeg" name="AccruedInterest" />
	<quote set="interestLeg" name="FundingOnRealizedTradingGains" />
	<quote set="fxSimpleOption" name="ImpliedVolatility" precision="8"/>
	<quote set="fxDigitalOption" name="ImpliedVolatility" precision="8"/>
	<quote set="fxBarrierOption" name="ImpliedVolatility" precision="8"/>
	<quote set="fxAverageRateOption" name="ImpliedVolatility" precision="8"/>
	<quote set="fxSimpleOption" name="CurrentFXRate" precision="8"/>
	<quote set="fxDigitalOption" name="CurrentFXRate" precision="8"/>
	<quote set="fxBarrierOption" name="CurrentFXRate" precision="8"/>
	<quote set="fxAverageRateOption" name="CurrentFXRate" precision="8"/>
	<quote set="equityLeg" name="EquityAccrual" />
	<quote set="equityLeg" name="RealizedTradingGains" />
	<quote set="equityLeg" name="InterestOnRealizedGains" />
	<quote set="equityLeg" name="AccruedCoupon" underlying="bonds"/>
	<quote set="varianceLeg" name="RealizedVariance" precision="8"/>
	<quote set="varianceLeg" name="ImpliedVolatility" precision="8"/>
	<quote set="equityOption" name="ImpliedVolatility" precision="8"/>
	<quote set="equityOption" name="CalculatedStrike" precision="8" strike="dswg:AssetSwapStrike" />
</xsl:variable>
<!--
	<quote set="singleUnderlyer" name="RealizedTradingGains" />
	<quote set="singleUnderlyer" name="InterestOnRealizedGains" />
	<quote set="basketConstituent" name="RealizedTradingGains" />
	<quote set="basketConstituent" name="InterestOnRealizedGains" />
-->

<!-- the above to a node set -->
<xsl:variable name="required.quotations" select="common:node-set($required.quotations.tree.frag)"/>

<!-- DSWG.010: ensure that required valuations are present -->
<xsl:variable name="optional.quotations.tree.frag">
	<quote set="basketConstituent" name="CurrentMarketValue" />
	<quote set="singleUnderlyer" name="CurrentMarketValue" />
</xsl:variable>

<!-- the above to a node set -->
<xsl:variable name="optional.quotations" select="common:node-set($optional.quotations.tree.frag)"/>

<!-- DSWG.010 : ensure that required valuations are present -->

<!-- required valuations elements -->
<xsl:template match="dswg:positionSummary|dswg:trade|fpml:swapStream|fpml:interestLeg|fpml:equityLeg|fpml:creditDefaultSwap|dswg:creditDefaultSwapBasket|fpml:varianceLeg|fpml:fxSimpleOption|fpml:fxDigitalOption|fpml:fxBarrierOption|fpml:fxAverageRateOption|dswg:referenceInformation" mode="val-valuations" >
	<xsl:if test="not(dswg:valuations)">
		<xsl:variable name="set" select="local-name(.)" />
		<xsl:variable name="affected">
			<xsl:for-each select="$required.quotations/*[@set=$set]">
				<xsl:if test="position() &gt; 1">, </xsl:if>
				<xsl:value-of select="@name"/>
			</xsl:for-each>
		</xsl:variable>
(DSWG.010) ERROR - Missing valuation structure in <xsl:apply-templates select="." mode="location"/>; the following measures are affected: <xsl:value-of select="$affected"/></xsl:if>
	<xsl:apply-templates mode="val-valuations" />
</xsl:template>

<xsl:template match="node()" mode="val-valuations" />

<!-- no required valuation element at this level, but some in children -->
<xsl:template match="fpml:swap|fpml:equitySwap|fpml:underlyer|fpml:basket|dswg:generalTerms" mode="val-valuations" >
	<xsl:apply-templates mode="val-valuations" />
</xsl:template>

<!-- details of valuation elements -->
<xsl:template match="dswg:valuations" mode="val-calculations" >
	<xsl:param name="set" select="local-name(..)" />
	<xsl:variable name="this" select="." />
	<xsl:variable name="actual.quotes" select="dswg:quotation" />
	<xsl:variable name="req.quotes" select="$required.quotations/*[@set=$set]"/>
	<xsl:variable name="product" select="local-name(../*[2])"/>
	<xsl:if test="count($req.quotes) &gt; 0">
		<xsl:for-each select="$required.quotations/*[@set=$set]">
			<xsl:variable name="measure" select="@name" />
			<xsl:variable name="strike" select="@strike" />
			<!-- default precision is 2 -->
			<xsl:variable name="prec">
				<xsl:choose>
					<xsl:when test="@precision"><xsl:value-of select="@precision"/></xsl:when>
					<xsl:otherwise>2</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="pct" select="@percentage"/>
			<xsl:variable name="stk.ok" select="not($strike) or ($this/../../fpml:equityOption/fpml:strike/@xsi:type=$strike)" />
			<xsl:variable name="und" select="@underlying" />
			<xsl:variable name="und.ok" select="not($und) or boolean($this/../fpml:underlyer//*[local-name(.) = $und]) or (($und = 'bonds') and boolean($this/../fpml:underlyer//*[local-name(.) = 'bond' or local-name(.)='convertibleBond']))" />
			<xsl:variable name="prod" select="@product" />
			<xsl:variable name="except" select="@except" />
			<xsl:variable name="prod.ok" select="($prod and ($prod = $product)) or ($except and not($product=$except)) or (not ($prod) and not($except))"/>
			<xsl:variable name="act.quote" select="$actual.quotes[fpml:measureType=$measure]"/>
			<xsl:if test="$prod.ok and $und.ok and $stk.ok and not($act.quote)">
(DSWG.011) ERROR - Missing quotation [<xsl:value-of select="$measure"/>] for <xsl:value-of select="$set" /> in <xsl:apply-templates select="$this" mode="location"/></xsl:if>
			<xsl:apply-templates select="$act.quote/fpml:value" mode="check-precision">
				<xsl:with-param name="prec" select="$prec"/>
				<xsl:with-param name="label" select="$measure"/>
				<xsl:with-param name="pct" select="$pct"/>
			</xsl:apply-templates>
		</xsl:for-each>
	</xsl:if>
	<!-- verify that no extra measures have crept in -->
	<xsl:variable name="required.quotes" select="$required.quotations/*[@set=$set]" />
	<xsl:variable name="optional.quotes" select="$optional.quotations/*[@set=$set]" />
	<xsl:variable name="allowed.quotes" select="$optional.quotes | $required.quotes"/>
	<xsl:for-each select="$actual.quotes">
		<xsl:variable name="measure" select="fpml:measureType"/>
		<xsl:if test="not($allowed.quotes[@name=$measure])">
(DSWG.015) - WARNING: Unexpected quotation [<xsl:value-of select="$measure"/>] for <xsl:value-of select="$set" /> in <xsl:apply-templates select="$this" mode="location"/></xsl:if>
	</xsl:for-each>
</xsl:template>


<!-- check quotation contents -->
<xsl:template match="dswg:valuations" mode="val-quotations" >
	<xsl:apply-templates select="dswg:quotation" mode="val-quotation" />
	<xsl:apply-templates select="dswg:quotation[measureType='NPVLocalCurrency']" mode="val-quotation-ccy" />
</xsl:template>

<xsl:template match="dswg:quotation" mode="val-quotation" >
	<xsl:if test="not(fpml:value)">
(DSWG.012) ERROR - Missing quotation value in <xsl:apply-templates select="." mode="location" />, measure = <xsl:value-of select="fpml:measureType"/>.
	</xsl:if>
	<xsl:if test="not(fpml:measureType)">
(DSWG.013) ERROR - Missing measure type in <xsl:apply-templates select="." mode="location" />, value = <xsl:value-of select="fpml:value"/>.
	</xsl:if>
</xsl:template>

<xsl:template match="dswg:quotation" mode="val-quotation-ccy" >
	<xsl:if test="not(fpml:currency)">
(DSWG.014) ERROR - Missing quotation currency in <xsl:apply-templates select="." mode="location" />, measure = <xsl:value-of select="fpml:measureType"/>.
	</xsl:if>
</xsl:template>


<!-- scheduled-dates related rules -->

<xsl:template match="dswg:positionSummary|dswg:trade|fpml:swap|fpml:equitySwap" mode="val-scheduled-dates" >
	<xsl:apply-templates mode="val-scheduled-dates"/>
</xsl:template>

<!-- required scheduledDates elements -->
<xsl:template match="dswg:positionSummary[fpml:equitySwap]|fpml:swapStream|fpml:interestLeg|fpml:equityLeg|fpml:creditDefaultSwap" mode="val-scheduled-dates" >
	<xsl:variable name="elem" select="local-name(.)"/>
	<xsl:variable name="set">
		<xsl:choose>
			<xsl:when test="$elem = 'positionSummary'">
				<xsl:value-of select="local-name(./*[2])"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$elem" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:if test="not(dswg:scheduledDates)">
		<xsl:variable name="affected">
			<xsl:for-each select="$required.scheduled.dates/*[@set=$set]">
				<xsl:if test="position() &gt; 1">, </xsl:if>
				<xsl:value-of select="@name"/>
			</xsl:for-each>
		</xsl:variable>
(DSWG.020) ERROR - Missing scheduled dates in <xsl:apply-templates select="." mode="location"/>; the following dates are affected: <xsl:value-of select="$affected"/></xsl:if>
	<xsl:apply-templates mode="val-scheduled-dates" />
</xsl:template>

<xsl:template match="node()" mode="val-scheduled-dates" />

<!-- list of required scheduled dates, by parent element name -->
<!-- corresponds to table of required scheduled dates in the DSWG specification document -->
<!-- the commented out dates may not always exist, therefore they have been
     removed from validation temporarily.
     We would like to see if there is a way to determine if this information
     is available -->
<xsl:variable name="required.scheduled.dates.tree.frag">
	<date set="swapStream" name="FirstPayment" />
	<date set="equitySwap" name="FinalPayment" />
	<date set="equityLeg" name="PreviousPayment" />
	<date set="interestLeg" name="PreviousPayment" />
	<!--
	<date set="interestLeg" name="PreviousReset" amount="1" warning="1" precision="8" pct="true"/>
	-->
	<date set="swapStream" name="PreviousReset" filter="float" amount="1" warning="1" precision="8" pct="true"/>
	<date set="creditDefaultSwap" name="FirstPayment" />
	<date set="creditDefaultSwap" name="AccrualStart" />
	<date set="creditDefaultSwapBasket" name="FirstPayment" />
	<date set="creditDefaultSwapBasket" name="AccrualStart" />
</xsl:variable>

<xsl:variable name="allowed.scheduled.dates.tree.frag">
	<date set="swapStream" name="PreviousPayment" />
	<date set="swapStream" name="NextPayment" />
	<date set="swapStream" name="NextReset" />
	<date set="equityLeg" name="PreviousReset" />
	<date set="equityLeg" name="NextReset" />
	<date set="equityLeg" name="NextPayment" />
	<date set="interestLeg" name="PreviousReset" amount="1" warning="1" precision="8" pct="true"/>
	<date set="interestLeg" name="NextPayment" />
	<date set="interestLeg" name="PreviousReset" />
</xsl:variable>

<!-- the above to a node set -->
<xsl:variable name="required.scheduled.dates" select="common:node-set($required.scheduled.dates.tree.frag)"/>
<xsl:variable name="allowed.scheduled.dates" select="common:node-set($allowed.scheduled.dates.tree.frag) | $required.scheduled.dates"/>



<xsl:template match="dswg:scheduledDates" mode="val-calculations" >
	<xsl:param name="parent" select="local-name(..)" />
	<!-- set name is product name for position summary -->
	<xsl:variable name="set">
		<xsl:choose>
			<xsl:when test="$parent = 'positionSummary'">
				<xsl:value-of select="local-name(../*[2])"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$parent" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="this" select="." />
	<xsl:variable name="actual.scheduled.dates" select="dswg:scheduledDate" />
	<xsl:for-each select="$required.scheduled.dates/*[@set=$set]">
		<xsl:variable name="type" select="@name" />
		<xsl:variable name="prec" select="@precision" />
		<xsl:variable name="pct" select="@pct" />
		<xsl:variable name="is.float" select="boolean($this/../fpml:resetDates)"/>
		<xsl:variable name="act" select="$actual.scheduled.dates[dswg:type=$type]"/>
		<xsl:variable name="skipped" select="(@filter='float' and not($is.float))"/>
		<xsl:if test="not($act) and not($skipped)">
(DSWG.020) <xsl:if test="@warning=1">WARNING - </xsl:if>Missing scheduled date [<xsl:value-of select="$type"/>] in <xsl:apply-templates select="$this" mode="location"/></xsl:if>
		<xsl:if test="(@amount) and not($act/dswg:amount) and not($skipped)">
(DSWG.022) ERROR - Missing amount for scheduled date [<xsl:value-of select="$type"/>] in <xsl:apply-templates select="$this" mode="location"/></xsl:if>
		<xsl:if test="$prec">
			<xsl:apply-templates mode="check-precision" select="$act/dswg:amount">
				<xsl:with-param name="prec" select="$prec"/>
				<xsl:with-param name="label" select="$type"/>
				<xsl:with-param name="pct" select="$pct"/>
			</xsl:apply-templates>
		</xsl:if>
	</xsl:for-each>

	<xsl:variable name="allowed.sd" select="$allowed.scheduled.dates/*[@set=$set]" />
	<xsl:for-each select="$actual.scheduled.dates">
		<xsl:variable name="type" select="dswg:type" />
		<xsl:if test="not($allowed.sd[@name=$type])">
(DSWG.021) WARNING: unexpected scheduled date [<xsl:value-of select="$type"/>] in <xsl:apply-templates select="$this" mode="location"/></xsl:if>
	</xsl:for-each>

</xsl:template>




<!-- DSWG.101 : validate adjustable dates -->

<!-- OK if adjustable date(s) -->
<xsl:template match="fpml:adjustableDate | fpml:adjustableDates" mode="val-adj-date" />

<!-- Bad if not adjustable date(s) -->
<xsl:template match="fpml:*" mode="val-adj-date" >
(DSWG.101) ERROR - Illegal date structure [<xsl:value-of select="local-name(.)"/>] in <xsl:apply-templates select=".." mode="location"/>
</xsl:template>

<!-- end of DSWG.101 -->


<!-- DSWG.102 : validate equity notionals -->

<!-- OK if explicit amount -->
<xsl:template match="fpml:notionalAmount" mode="val-equity-notional" />

<!-- bad otherwise -->
<xsl:template match="fpml:*" mode="val-equity-notional" >
(DSWG.102) ERROR - Illegal notional structure [<xsl:value-of select="local-name(.)"/>] in <xsl:apply-templates select=".." mode="location"/>
</xsl:template>

<!-- end of DSWG.102 -->


<!-- DSWG.103 : validate equity principal exchange amounts -->

<!-- OK if explicit amount -->
<xsl:template match="fpml:principalAmount" mode="val-equity-principal-exchange-amount" />

<!-- bad otherwise -->
<xsl:template match="fpml:*" mode="val-equity-principal-exchange-amount" >
(DSWG.103) ERROR - Illegal principal exchange amount [<xsl:value-of select="local-name(.)"/>] in <xsl:apply-templates select=".." mode="location"/>
</xsl:template>

<!-- end of DSWG.103 -->

<!-- DSWG.104 : prohibit business centers references -->

<!-- OK if explicit list -->
<xsl:template match="node()" mode="val-business-centers" />

<!-- bad otherwise -->
<xsl:template match="fpml:businessCentersReference" mode="val-business-centers" >
(DSWG.104) ERROR - Illegal business centers reference in <xsl:apply-templates select=".." mode="location"/>
</xsl:template>

<!-- end of DSWG.104 -->

<!-- DSWG.105 : require business centers -->
<xsl:template match="node()[fpml:businessCenters]" mode="require-business-centers" />

<xsl:template match="node()[fpml:adjustableDate]" mode="require-business-centers" >
	<xsl:apply-templates mode="require-business-centers" select="fpml:adjustableDate" />
</xsl:template>

<xsl:template match="node()[fpml:dateAdjustments]" mode="require-business-centers" >
	<xsl:apply-templates mode="require-business-centers" select="fpml:dateAdjustments" />
</xsl:template>

<xsl:template match="node()" mode="require-business-centers" >
(DSWG.105) ERROR - Missing business centers in <xsl:apply-templates select="." mode="location"/>
</xsl:template>

<!-- DSWG.106 : require non-null -->
<xsl:template match="node()[string-length(text()) &gt; 0]" mode="require-data" />

<xsl:template match="node()" mode="require-data" >
(DSWG.106) ERROR - Missing data in <xsl:apply-templates select="." mode="location"/>
</xsl:template>

<!-- DSWG.107 : validate floating rate index -->
<xsl:template match="fpml:floatingRateIndex" mode="validate-index" >
	<xsl:variable name="idx" select="text()"/>
	<xsl:variable name="ccy" select="substring($idx,1,3)"/>

	<xsl:choose>
		<xsl:when test="not($float.idx.ccys/val[.=$ccy])">
(DSWG.107) WARNING - Unknown index currency <xsl:value-of select="$ccy"/> in index <xsl:value-of select="$idx"/>, at <xsl:apply-templates select="." mode="location"/>.</xsl:when>
		<xsl:when test="not($float.indexes/val[.=$idx])">
(DSWG.107) WARNING - Unknown index value <xsl:value-of select="$idx"/>, at <xsl:apply-templates select="." mode="location"/>.</xsl:when>
	</xsl:choose>
	<!--
	<xsl:if test="not(contains($idx, '-')) or not(contains(substring-after($idx,'-'),'-'))">
(DSWG.107) ERROR - Floating rate index should contain at least 2 '-' characters: please use ISDA approved values, at <xsl:apply-templates select="." mode="location"/>.</xsl:if>

	<xsl:if test="contains($idx, '_')">
(DSWG.107) ERROR - Floating rate index contains '_': please use ISDA approved values, at <xsl:apply-templates select="." mode="location"/>.</xsl:if>


	<xsl:variable name="strip" select="translate($idx, 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_ ()/.=:', '')" />

	<xsl:if test="string-length($strip) &gt; 0">
(DSWG.107) ERROR - Floating rate index contains illegal characters '<xsl:value-of select="$strip"/>': please use ISDA approved values with no padding, at <xsl:apply-templates select="." mode="location"/>.</xsl:if>
-->
</xsl:template>

<xsl:template match="node()" mode="check-precision" >
	<xsl:param name="prec" select="2" />
	<xsl:param name="label" select="'field'" />
	<xsl:param name="pct" select="false" />
	<xsl:variable name="dec" select="substring-after(., '.')"/>
	<xsl:if test="not(string-length($dec)=$prec)">
(DSWG.111) ERROR - Precision of <xsl:value-of select="$label"/> should be <xsl:value-of select="$prec"/>, at <xsl:apply-templates select="." mode="location"/>; current value is <xsl:value-of select="."/>.</xsl:if>
	<xsl:if test="$pct and (. &gt; 1.0)">
(DSWG.112) ERROR - <xsl:value-of select="$label"/> should be expressed as a decimal, at <xsl:apply-templates select="." mode="location"/>; current value is <xsl:value-of select="."/>.</xsl:if>
</xsl:template>


<xsl:template match="node()" mode="validate-currency-schedule" >
	<xsl:apply-templates mode="validate-currency-precision" select="fpml:initialValue"/>
	<xsl:apply-templates mode="validate-currency-precision" select="fpml:step/fpml:stepValue"/>
</xsl:template>

<xsl:template match="node()" mode="validate-currency-precision" >
	<xsl:variable name="dec" select="substring-after(., '.')"/>
	<xsl:if test="not(string-length($dec)=2)">
(DSWG.108) ERROR - Monetary amount precision should be 2, at <xsl:apply-templates select="." mode="location"/>; current value is <xsl:value-of select="."/>.</xsl:if>
</xsl:template>


<xsl:template match="node()" mode="validate-rate-schedule" >
	<xsl:apply-templates mode="validate-rate" select="fpml:initialValue"/>
	<xsl:apply-templates mode="validate-rate" select="fpml:step/fpml:stepValue"/>
</xsl:template>

<xsl:template match="node()" mode="validate-rate" >
	<xsl:apply-templates select="." mode="validate-rate-precision"/>
	<xsl:if test=". &gt; 1.0">
(DSWG.110) ERROR - Rates/spreads should be expressed as a decimal (e.g., 0.05000000), at <xsl:apply-templates select="." mode="location"/>; current value is <xsl:value-of select="."/>.</xsl:if>
</xsl:template>

<xsl:template match="node()" mode="validate-rate-precision" >
	<xsl:variable name="dec" select="substring-after(., '.')"/>
	<xsl:if test="not(string-length($dec)=8)">
(DSWG.109) ERROR - Rate/weight precision should be 8, at <xsl:apply-templates select="." mode="location"/>.</xsl:if>
</xsl:template>


<!-- ****  exchange and underlyer related rules ****  -->
<xsl:template match="fpml:underlyer" mode="val-underlyers" >
	<xsl:param name="dividend" select="1"/>
	<xsl:param name="full" select="1"/>
	<xsl:apply-templates select="*" mode="val-underlyers" >
		<xsl:with-param name="dividend" select="$dividend"/>
		<xsl:with-param name="full" select="$full"/>
	</xsl:apply-templates>
</xsl:template>

<!-- validate a single underlyer -->
<xsl:template match="fpml:singleUnderlyer|fpml:basketConstituent" mode="val-underlyers" >
	<xsl:param name="full" select="1"/>
	<xsl:param name="dividend" select="1"/>

	<xsl:apply-templates select="*[1]" mode="val-security-id" />
	<xsl:if test="$full">
		<xsl:apply-templates select="*[1]" mode="val-bond" />
	</xsl:if>
	<xsl:apply-templates select="*[1]" mode="val-underlyer-ccy" />
	<xsl:apply-templates select="*[1]" mode="val-underlyer-desc" />
	<xsl:if test="$dividend and not(ancestor::fpml:varianceLeg)">
		<xsl:apply-templates select="." mode="val-dividend-payout" />
		<xsl:apply-templates select="." mode="val-open-units" />
		<xsl:apply-templates select="." mode="val-prices" />
	</xsl:if>
</xsl:template>

<!-- validate all constituents of a basket -->
<xsl:template match="fpml:basket" mode="val-underlyers" >
	<xsl:param name="full" select="true()"/>
	<xsl:param name="dividend" select="1"/>
	<xsl:apply-templates select="fpml:basketConstituent" mode="val-underlyers" >
		<xsl:with-param name="dividend" select="$dividend"/>
		<xsl:with-param name="full" select="$full"/>
	</xsl:apply-templates>
</xsl:template>

<!-- list the US stock exchanges -->
<xsl:variable name="us.exchanges.tree.frag">
	<exchg name="NYSE" />
	<exchg name="AMEX" />
	<exchg name="NASDAQ" />
	<exchg name="OTC US" />
	<exchg name="Pink Sheets" />
	<exchg name="PSE" />
	<exchg name="Boston" />
	<exchg name="Cincinnati" />
</xsl:variable>
<xsl:variable name="us.exchanges" select="common:node-set($us.exchanges.tree.frag)" />

<!-- list the instrument ID schemes -->
<xsl:variable name="instrument-id-schemes-TF">
	<idType name="CUSIP" scheme="http://www.fpml.org/spec/2002/instrument-id-CUSIP-1-0" />
	<!-- Committee on Uniform Securities Identification Procedures -->
	<idType name="ISIN" scheme="http://www.fpml.org/spec/2002/instrument-id-ISIN-1-0" />
	<!-- International Securities Identification Number -->
	<idType name="RIC" scheme="http://www.fpml.org/spec/2003/instrument-id-Reuters-RIC-1-0" />
	<!-- Reuters Instrument Code (RIC) -->
	<idType name="RED" scheme="http://www.fpml.org/spec/2003/instrument-id-RED-pair-1-0" />
	<!-- RED pair code -->
	<idType name="SEDOL" scheme="http://www.fpml.org/spec/2002/instrument-id-SEDOL-1-0" />
	<!-- London Stock Exchange Daily Official List -->
	<idType name="Sicovam" scheme="http://www.fpml.org/spec/2002/instrument-id-Sicovam-1-0" />
	<!-- Sicovam code -->
</xsl:variable>
<xsl:variable name="instrument-id-schemes" select="common:node-set($instrument-id-schemes-TF)" />
<xsl:variable name="cusip" select="$instrument-id-schemes/*[@name='CUSIP']/@scheme" />
<xsl:variable name="isin" select="$instrument-id-schemes/*[@name='ISIN']/@scheme" />
<xsl:variable name="sedol" select="$instrument-id-schemes/*[@name='SEDOL']/@scheme" />

<!-- validate security id -->
<!-- test that at least one of ISIN, SEDOL, or CUSIP is present -->
<xsl:template match="fpml:bond" mode="val-security-id" >
	<xsl:choose>
		<xsl:when test="fpml:instrumentId[@instrumentIdScheme = $cusip]" />
		<xsl:when test="fpml:instrumentId[@instrumentIdScheme = $isin]" />
		<xsl:when test="fpml:instrumentId[@instrumentIdScheme = $sedol]" />
		<xsl:otherwise>
(DSWG.201) ERROR - Missing/invalid instrument ID for bond (requires CUSIP, ISIN, or SEDOL) in <xsl:apply-templates select="." mode="location"/>.
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="fpml:bond|fpml:convertibleBond" mode="val-bond" >
	<xsl:if test="not(fpml:parValue)">
(DSWG.223) ERROR - Missing par value for bond in <xsl:apply-templates select="." mode="location"/>.</xsl:if>

	<xsl:if test="not(../dswg:numberOfUnderlyingSecurities)">
(DSWG.224) ERROR - Missing number of underlying securities for bond in <xsl:apply-templates select="." mode="location"/>.</xsl:if>

</xsl:template>

<xsl:template match="node()" mode="val-bond" />

<!-- sec IDs for indices are not checked, per Bob Du Fief -->
<xsl:template match="fpml:index" mode="val-security-id" />

<!-- test that CUSIP is present for US equities, ISIN or SEDOL for non-US -->

<xsl:template match="fpml:equity" mode="val-security-id" >
	<xsl:variable name="exchange" select="fpml:exchangeId" />
	<!-- are we on the list of US exchanges? -->
	<xsl:variable name="isUs" select="boolean($us.exchanges/*[@name=$exchange])" />

	<xsl:choose>
		<xsl:when test="not($exchange)" >
(DSWG.202) ERROR - Missing exchange for equity in <xsl:apply-templates select="." mode="location"/>
		</xsl:when>
		<!-- ok if is US and has a CUSIP -->
		<xsl:when test="$isUs and fpml:instrumentId[@instrumentIdScheme = $cusip]" />
		<xsl:when test="$isUs" >
		<!-- if is US we have a problem - no CUSIP -->
(DSWG.203) ERROR - Missing/invalid instrument ID for US equity (requires CUSIP) in <xsl:apply-templates select="." mode="location"/>.
		</xsl:when>
		<!-- ok if we have an ISIN, CUSIP, or SEDOL -->
		<xsl:when test="fpml:instrumentId[@instrumentIdScheme = $isin]" />
		<xsl:when test="fpml:instrumentId[@instrumentIdScheme = $sedol]" />
		<xsl:otherwise>
(DSWG.204) ERROR - Missing instrument ID for non-US equity (requires ISIN or SEDOL) in <xsl:apply-templates select="." mode="location"/>.
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- unknown underlyer type -->
<xsl:template match="node()" mode="val-security-id" >
	<!--
(DSWG.205) Unknown TRS underlyer type <xsl:value-of select="local-name(.)" /> in <xsl:apply-templates select="." mode="location"/>.
-->
</xsl:template>

<!-- DSWG.206 validate security currency  -->
<!-- check for missing currency -->
<xsl:template match="node()[string-length(fpml:currency) &gt; 0]" mode="val-underlyer-ccy" />

<!-- missing currency -->
<xsl:template match="node()" mode="val-underlyer-ccy" >
(DSWG.206) ERROR - Missing TRS underlyer currency in <xsl:apply-templates select="." mode="location"/>.
</xsl:template>

<!-- DSWG.221 validate security description -->
<!-- check for missing description -->
<xsl:template match="node()[string-length(fpml:description) &gt; 0]" mode="val-underlyer-desc" />

<!-- missing currency -->
<xsl:template match="node()" mode="val-underlyer-desc" >
(DSWG.221) ERROR - Missing TRS underlyer description (issuer name) in <xsl:apply-templates select="." mode="location"/>.
</xsl:template>

<!-- missing open units ? -->

<xsl:template match="fpml:singleUnderlyer[fpml:openUnits]" mode="val-open-units" />
<xsl:template match="node()" mode="val-open-units" />

<xsl:template match="fpml:singleUnderlyer" mode="val-open-units" >
(DSWG.227) ERROR - TRS single underlyer open units are missing, <xsl:apply-templates select="." mode="location"/>.
</xsl:template>

<!-- missing prices ? -->

<xsl:template match="node()[fpml:bond | fpml:convertibleBond]" mode="val-prices" >
	<xsl:apply-templates select="." mode="val-dirty-prices" />
	<xsl:apply-templates select="." mode="val-clean-prices" />
	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:amount"/>
	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:accruedInterestPrice"/>
</xsl:template>

<xsl:template match="node()" mode="val-prices" >
	<xsl:apply-templates select="." mode="val-dirty-prices" />
	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:amount"/>
	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:accruedInterestPrice"/>
</xsl:template>

<xsl:template match="node()" mode="val-dirty-prices" >
	<xsl:variable name="und" select="fpml:underlyerPrice | dswg:underlyerPrice" />
	<xsl:if test="not($und/fpml:netPrice)">
(DSWG.230) ERROR - TRS single underlyer net (dirty) reset price is missing, <xsl:apply-templates select="." mode="location"/>.</xsl:if>
	<xsl:if test="not(dswg:currentMarketPrice/fpml:netPrice)">
(DSWG.231) ERROR - TRS single underlyer net (dirty) current market price is missing, <xsl:apply-templates select="." mode="location"/>.</xsl:if>
</xsl:template>

<xsl:template match="node()" mode="val-clean-prices" >
	<xsl:variable name="und" select="fpml:underlyerPrice | dswg:underlyerPrice" />
	<xsl:if test="not($und/dswg:cleanNetPrice)">
(DSWG.232) ERROR - TRS single underlyer net clean reset price is missing, <xsl:apply-templates select="." mode="location"/>.</xsl:if>
	<xsl:if test="not(dswg:currentMarketPrice/dswg:cleanNetPrice)">
(DSWG.233) ERROR - TRS single underlyer net clean current market price is missing, <xsl:apply-templates select="." mode="location"/>.</xsl:if>
</xsl:template>



<!-- missing dividend payout ? -->

<xsl:template match="node()[fpml:equity and not(fpml:dividendPayout)]" mode="val-dividend-payout" >
(DSWG.207) ERROR - Missing TRS dividend payout in <xsl:apply-templates select="." mode="location"/>.
</xsl:template>

<xsl:template match="node()[fpml:dividendPayout]" mode="val-dividend-payout" >
	<xsl:if test="fpml:dividendPayout/fpml:dividendPayoutRatio &gt; 1">
(DSWG.207b) WARNING - TRS dividend payout is greater than 100% - please specify dividend payouts as decimal fractions, not percentages, at <xsl:apply-templates select="." mode="location"/>.
	</xsl:if>

	<xsl:apply-templates select="fpml:dividendPayout/fpml:dividendPayoutRatio" mode="validate-rate-precision"/>
	<xsl:apply-templates select="fpml:dividendPayout//fpml:amount" mode="validate-currency-precision"/>
</xsl:template>

<xsl:template match="node()" mode="val-dividend-payout" />


<!-- missing dividend amount ? -->
<xsl:template match="fpml:return" mode="val-dividend-amount" >
	<!-- applies only if equity underlyers are present -->
	<xsl:if test="../fpml:underlyer//fpml:equity">
		<xsl:if test="not(fpml:dividendConditions/fpml:dividendAmount)">
(DSWG.208) WARNING: Missing TRS dividend amount in <xsl:apply-templates select="." mode="location"/>.
		</xsl:if>
		<xsl:if test="not(fpml:dividendConditions/fpml:dividendPaymentDate)">
(DSWG.209) WARNING: Missing TRS dividend paymentDate in <xsl:apply-templates select="." mode="location"/>.
		</xsl:if>
	</xsl:if>
</xsl:template>

<!-- validate commissions in price structures -->
<xsl:template mode="val-commission" match="fpml:initialPrice|fpml:valuationPriceInterim|fpml:valuationPriceFinal">
	<xsl:if test="not(fpml:commission)">
(DSWG.210) WARNING: Missing TRS commission in <xsl:apply-templates select="." mode="location"/>.
	</xsl:if>
	<xsl:apply-templates select="fpml:commission" mode ="val-commission-per-trade"/>
</xsl:template>

<!-- validate commission per trade -->
<!-- provided, ok -->
<xsl:template mode="val-commission-per-trade" match="fpml:commission[dswg:commissionPerTrade]" />

<!-- not provided, warn -->
<xsl:template mode="val-commission-per-trade" match="fpml:commission" >
(DSWG.211) WARNING: Missing TRS commission per trade in <xsl:apply-templates select="." mode="location"/>.
</xsl:template>


<!-- ******* validate CDS ******* -->
<xsl:template match="fpml:creditDefaultSwap" mode="val-product" >
	<xsl:apply-templates mode="val-fees" select="fpml:feeLeg" />
	<xsl:if test="not(fpml:generalTerms/fpml:dateAdjustments/fpml:businessDayConvention/text())">
DSWG.408 ERROR - Missing CDS Business Day Convention in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:apply-templates select="fpml:generalTerms/fpml:dateAdjustments" mode="require-business-centers" />

	<xsl:if test="not(fpml:generalTerms/fpml:effectiveDate/fpml:unadjustedDate)">
DSWG.402 ERROR - Missing CDS Effective Date in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(fpml:generalTerms/fpml:scheduledTerminationDate/fpml:adjustableDate/fpml:unadjustedDate)">
DSWG.411 ERROR - Missing CDS Termination Date in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(fpml:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:fixedRate)">
DSWG.403 - WARNING - Missing CDS Fixed Rate in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:apply-templates mode="validate-rate" select="fpml:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:fixedRate"/>
	<xsl:if test="not(fpml:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:dayCountFraction/text())">
DSWG.404 - WARNING - Missing CDS Day Count Fraction in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:if test="not(fpml:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:calculationAmount/fpml:amount)">
DSWG.405 - WARNING - Missing CDS Fixed Notional in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(fpml:protectionTerms/fpml:calculationAmount/fpml:amount)">
DSWG.406 ERROR - Missing CDS Floating Notional in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:apply-templates mode="validate-currency-precision" select="fpml:protectionTerms//fpml:amount"/>

	<xsl:variable name="refinfo" select="fpml:generalTerms/fpml:referenceInformation"/>
	<xsl:variable name="indexinfo" select="fpml:generalTerms/fpml:indexReferenceInformation"/>

	<xsl:if test="not($indexinfo) and not(fpml:generalTerms/fpml:referenceInformation/fpml:referenceEntity/fpml:entityName/text())">
DSWG.407 ERROR - Missing CDS Reference Entity Name in <xsl:apply-templates select="." mode="location"/></xsl:if>
	
	<xsl:if test="not($refinfo) and not($indexinfo)">
DSWG.409 ERROR - Missing CDS Reference Obligation in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:apply-templates select="fpml:generalTerms/fpml:referenceInformation/fpml:referenceObligation/*[1]" mode="val-security-id" />

	<xsl:if test="not ($indexinfo) and not(fpml:physicalSettlementTerms|fpml:cashSettlementTerms)">
DSWG.410 ERROR - Missing CDS Settlement Terms in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:apply-templates select=".//fpml:restructuringType" mode="val-restructuring" />

	<xsl:if test="not(fpml:feeLeg/fpml:initialPayment/fpml:paymentAmount)">
DSWG.420 ERROR - Missing CDS Upfront Fee (provide 0 fee if not applicable) in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(fpml:feeLeg/fpml:initialPayment/fpml:adjustablePaymentDate) and (fpml:feeLeg/fpml:initialPayment/fpml:paymentAmount/fpml:amount != 0)">
DSWG.421 ERROR - Missing CDS Upfront Fee payment date in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:apply-templates mode="validate-currency-precision" select="fpml:feeLeg//fpml:amount"/>

	<!-- CDS index product -->
	<xsl:if test="$indexinfo and fpml:feeLeg/fpml:singlePayment">
DSWG.412 ERROR - Single payment not permitted for CDS Index in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="$indexinfo and fpml:protectionTerms/fpml:creditEvents/fpml:restructuring">
DSWG.413 ERROR - Restructuring event not permitted for CDS Index in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:variable name="periodic" select="fpml:feeLeg/fpml:periodicPayment"/>
	<xsl:if test="$indexinfo and ($periodic/fpml:paymentFrequency|$periodic/fpml:rollConvention|$periodic/fpml:firstPaymentDate)">
DSWG.414 ERROR - Periodic payment frequency, roll convention, and first payment date are not permitted for CDS Index in <xsl:apply-templates select="." mode="location"/></xsl:if>

</xsl:template>

	<!-- CDS basket product -->

<xsl:template match="dswg:creditDefaultSwapBasket" mode="val-product" >
	<xsl:apply-templates mode="val-fees" select="dswg:feeLeg" />
	<xsl:if test="not(dswg:generalTerms/dswg:dateAdjustments/fpml:businessDayConvention/text())">
DSWG.408 ERROR - Missing CDS Business Day Convention in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:apply-templates select="dswg:generalTerms/dswg:dateAdjustments" mode="require-business-centers" />

	<xsl:if test="not(dswg:generalTerms/dswg:effectiveDate/fpml:unadjustedDate)">
DSWG.402 ERROR - Missing CDS Effective Date in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(dswg:generalTerms/dswg:scheduledTerminationDate/fpml:adjustableDate/fpml:unadjustedDate)">
DSWG.411 ERROR - Missing CDS Termination Date in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(dswg:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:fixedRate)">
DSWG.403 - WARNING - Missing CDS Fixed Rate in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:apply-templates mode="validate-rate" select="dswg:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:fixedRate"/>
	<xsl:if test="not(dswg:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:dayCountFraction/text())">
DSWG.404 - WARNING - Missing CDS Day Count Fraction in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:if test="not(dswg:feeLeg/fpml:periodicPayment/fpml:fixedAmountCalculation/fpml:calculationAmount/fpml:amount)">
DSWG.405 - WARNING - Missing CDS Fixed Notional in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(dswg:protectionTerms/fpml:calculationAmount/fpml:amount)">
DSWG.406 ERROR - Missing CDS Floating Notional in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:apply-templates mode="validate-currency-precision" select="dswg:protectionTerms//fpml:amount"/>

	<xsl:for-each select="dswg:generalTerms/dswg:referenceInformation">
		<xsl:if test="not(fpml:referenceEntity/fpml:entityName/text())">
DSWG.407 ERROR - Missing CDS Reference Entity Name in <xsl:apply-templates select="." mode="location"/></xsl:if>
	
		<xsl:if test="not(fpml:referenceObligation)">
DSWG.409 ERROR - Missing CDS Reference Obligation in <xsl:apply-templates select="." mode="location"/></xsl:if>

		<xsl:apply-templates select="fpml:referenceObligation/*[1]" mode="val-security-id" />
	</xsl:for-each>

	<xsl:if test="not(dswg:physicalSettlementTerms|dswg:cashSettlementTerms)">
DSWG.410 ERROR - Missing CDS Settlement Terms in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:apply-templates select=".//fpml:restructuringType" mode="val-restructuring" />

	<xsl:if test="not(dswg:feeLeg/fpml:initialPayment/fpml:paymentAmount)">
DSWG.420 ERROR - Missing CDS Upfront Fee (provide 0 fee if not applicable) in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:if test="not(dswg:feeLeg/fpml:initialPayment/fpml:adjustablePaymentDate) and (dswg:feeLeg/fpml:initialPayment/fpml:paymentAmount/fpml:amount != 0)">
DSWG.421 ERROR - Missing CDS Upfront Fee payment date in <xsl:apply-templates select="." mode="location"/></xsl:if>

	<xsl:apply-templates mode="validate-currency-precision" select="dswg:feeLeg//fpml:amount"/>
</xsl:template>


<xsl:variable name="scheme.valsTF">
	<val scheme="restructuring" val="R"/>
	<val scheme="restructuring" val="ModR"/>
	<val scheme="restructuring" val="ModModR"/>
</xsl:variable>
<xsl:variable name="scheme.vals" select="common:node-set($scheme.valsTF)"/>


<xsl:template mode="val-restructuring" match="fpml:restructuringType">
	<xsl:apply-templates mode="val-scheme" select=".">
		<xsl:with-param name="scheme" select="'restructuring'" />
	</xsl:apply-templates>
</xsl:template>

<xsl:template mode="val-scheme" match="node()">
	<xsl:param name="scheme" />
	<xsl:variable name="this" select="." />
	<xsl:if test="not($scheme.vals/*[@scheme=$scheme and @val=$this])">
DSWG.050 - WARNING - unexpected <xsl:value-of select="$scheme" /> scheme value <xsl:value-of select="$this"/> at <xsl:apply-templates mode="location" select="$this"/>
	</xsl:if>
</xsl:template>

<xsl:template mode="val-fees" match="node()">
	<xsl:if test="not(fpml:initialPayment | fpml:singlePayment | fpml:periodicPayment)">
(DSWG.401) WARNING: Missing fees in <xsl:apply-templates select="." mode="location"/>.
	</xsl:if>
</xsl:template>

<!-- ******* validate FX ******* -->

<!-- ******* validate fx spot/fwd ******* -->
<xsl:template match="fpml:fxSingleLeg" mode="val-product" >
	<xsl:apply-templates mode="val-nonDeliv" select=".//fpml:nonDeliverableForward" />

	<xsl:if test="not(dswg:paymentDateAdjustments)">
(DSWG.502) WARNING - payment date adjustments not supplied at <xsl:apply-templates select="." mode="location"/>.
	</xsl:if>

	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:paymentAmount/fpml:amount"/>
	<xsl:apply-templates mode="validate-rate-precision" select=".//fpml:spotRate | .//fpml:rate | .//fpml:forwardPoints"/>
</xsl:template>

<!-- ******* validate fx option ******* -->
<xsl:template match="fpml:fxSimpleOption" mode="val-product" >
	<xsl:apply-templates mode="val-fx-option" select="."/>
</xsl:template>

<!-- ******* validate fx barrrier option ******* -->
<xsl:template match="fpml:fxBarrierOption" mode="val-product" >
	<xsl:apply-templates mode="val-fx-option" select="."/>
</xsl:template>

<!-- ******* validate fx option ******* -->
<xsl:template match="fpml:fxAverageRateOption" mode="val-product" >
	<xsl:apply-templates mode="val-fx-option" select="."/>
</xsl:template>

<!-- ******* validate fx option ******* -->
<xsl:template match="fpml:fxDigitalOption" mode="val-product" >
	<xsl:apply-templates mode="val-fx-option" select="."/>
</xsl:template>

<!-- ******* fx shared validation ******* -->
<xsl:template match="node()" mode="val-nonDeliv" >
	<xsl:if test="not(fpml:fixing/dswg:settlementRateOption)">
DSWG.501 ERROR - Settlement Rate Option not supplied for Non-deliverable forward/option</xsl:if>
</xsl:template>

<xsl:template match="node()" mode="val-fx-option" >
	<xsl:apply-templates mode="val-fx-premium" select="."/>
	<xsl:apply-templates mode="val-fx-exercise" select="."/>
	<xsl:apply-templates mode="val-nonDeliv" select=".//fpml:cashSettlementTerms" />

	<xsl:if test="not(dswg:expirationDateAdjustments)">
DSWG.503 WARNING - Expiration date adjustments not supplied for FX option</xsl:if>
	<xsl:apply-templates select="dswg:expirationDateAdjustments" mode="require-business-centers" />
	<xsl:if test="not(dswg:paymentDateAdjustments)">
DSWG.504 WARNING - Payment date adjustments not supplied for FX option</xsl:if>
	<xsl:apply-templates select="dswg:paymentDateAdjustments" mode="require-business-centers" />

	<xsl:apply-templates mode="validate-currency-precision" select=".//fpml:amount"/>
</xsl:template>

<!-- validate the FX premium terms -->
<xsl:template match="node()" mode="val-fx-premium" >
	<xsl:if test="not(fpml:fxOptionPremium)">
DSWG.520 ERROR - Premium not supplied for FX option</xsl:if>
</xsl:template>

<!-- validate the FX exercise terms -->
<xsl:template match="node()" mode="val-fx-exercise" >
	<xsl:if test="(fpml:exerciseStyle='Bermuda') and not(fpml:expiryDateTime/dswg:exerciseDate)">
DSWG.521 ERROR - Exercise dates not supplied for Bermuda FX option at <xsl:apply-templates mode="location" select="."/>. (Use FxBermudaExpiry type).</xsl:if>
</xsl:template>

<!-- **** FX shared templates -->

<xsl:template match="dswg:shortForm" mode="val-product" >
	<xsl:if test="not(../dswg:positionHeader/dswg:senderPartyReference)">
DSWG.802 ERROR - Sender party reference not supplied for Short Form product at <xsl:apply-templates mode="location" select="."/>. </xsl:if>
	<xsl:if test="not(../dswg:positionHeader/dswg:receiverPartyReference)">
DSWG.803 ERROR - Receiver party reference not supplied for Short Form product at <xsl:apply-templates mode="location" select="."/>. 
	</xsl:if>
</xsl:template>
<!-- *************** Reporting/display templates *********************** -->

<!-- display location -->
<xsl:template match="node()" mode="location" >
	<xsl:variable name="mytag" select="name(.)" /> 
	<xsl:variable name="mysibs" select="../*[local-name(.)=$mytag]" />
	<xsl:variable name="mypos"><xsl:number /></xsl:variable>
	<xsl:variable name="mypath">
		<xsl:choose>
			<xsl:when test="count($mysibs) &gt; 1">
				<xsl:value-of select="concat($mytag, '[', $mypos, ']')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$mytag"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:apply-templates mode="location" select=".."/>/<xsl:value-of select="$mypath"/>
</xsl:template>

<xsl:template match="fpml:FpML" mode="location" >Unknown position</xsl:template>

<xsl:template match="dswg:position" mode="location" >position #<xsl:value-of select="dswg:positionSummary/dswg:positionHeader/dswg:senderPositionIdentifier" />
</xsl:template>

<!-- product type -->

<xsl:template mode="validate-productType" match="*[not(fpml:productType)]">(DSWG.040) ERROR - product type is not specified.</xsl:template>


<xsl:template mode="validate-productType" match="dswg:creditDefaultSwapBasket[fpml:productType]" priority="10">
	<xsl:if test="not(fpml:productType = 'CDS-Basket')">(DSWG.041b) ERROR - supplied product type (<xsl:value-of select="fpml:productType"/>) does not match expected value (CDS-Basket) in <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<xsl:template mode="validate-productType" match="fpml:creditDefaultSwap[fpml:productType and fpml:generalTerms/fpml:indexReferenceInformation]" priority="10">
	<xsl:variable name="provided" select="fpml:productType" />
	<xsl:if test="$provided != 'CDS-Index'">(DSWG.041) ERROR - supplied product type (<xsl:value-of select="$provided"/>) does not match expected value (CDS-[Index]) in <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<xsl:template mode="validate-productType" match="fpml:creditDefaultSwap[fpml:productType]" priority="10">
	<xsl:variable name="provided" select="fpml:productType" />
	<xsl:variable name="front" select="substring-before($provided, '-')"/>
	<xsl:variable name="back" select="substring-after($provided, '-')"/>
	<!--
	<xsl:value-of select="concat('front:', $front, ', back: ', $back)"/>
	-->
	<xsl:if test="$front != 'CDS' or ($back != 'S' and $back != 'U')">(DSWG.041) ERROR - supplied product type (<xsl:value-of select="$provided"/>) does not match expected value (CDS-[U|S]) in <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<xsl:template mode="validate-productType" match="dswg:varianceSwap[fpml:productType]" priority="10">
	<xsl:variable name="provided" select="fpml:productType" />
	<xsl:variable name="front" select="substring-before($provided, '-')"/>
	<xsl:variable name="back" select="substring-after($provided, '-')"/>
	<!--
	<xsl:value-of select="concat('front:', $front, ', back: ', $back)"/>
	-->
	<xsl:if test="$front != 'VarianceSwap' and ($front != 'VolatilitySwap')">(DSWG.270) ERROR - supplied product type (<xsl:value-of select="$provided"/>) does not match expected value ([VolatilitySwap | VarianceSwap]) in <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<xsl:template mode="validate-productType" match="*[fpml:productType]">
	<xsl:variable name="provided" select="fpml:productType" />
	<xsl:variable name="needed"><xsl:apply-templates select="." mode="productType"/></xsl:variable>
	<xsl:if test="$provided != $needed">(DSWG.041) ERROR - supplied product type (<xsl:value-of select="$provided"/>) does not match expected value (<xsl:value-of select="$needed"/>) in <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<xsl:template mode="productType" match="fpml:swap">IRS-<xsl:apply-templates mode="swapType" select="."/><xsl:apply-templates select="." mode="swapUneven"/><xsl:apply-templates mode="swapFeatures"/></xsl:template>

<xsl:template mode="productType" match="fpml:swaption">IRSwaption-<xsl:apply-templates mode="exerType" select="."/>-<xsl:apply-templates mode="swapType" select="fpml:swap"/></xsl:template>

<!-- swap type -->
<xsl:template mode="swapType" match="fpml:swap[count(fpml:swapStream) != 2]">OddLegged</xsl:template>
<xsl:template mode="swapType" match="fpml:swap[count(fpml:swapStream) = 2]">
	<xsl:variable name="ccy1"><xsl:apply-templates select="fpml:swapStream[1]" mode="swapStrCcy" /></xsl:variable>
	<xsl:variable name="ccy2"><xsl:apply-templates select="fpml:swapStream[2]" mode="swapStrCcy" /></xsl:variable>
	<xsl:variable name="ff1"><xsl:apply-templates select="fpml:swapStream[1]" mode="swapStrFixFloat" /></xsl:variable>
	<xsl:variable name="ff2"><xsl:apply-templates select="fpml:swapStream[2]" mode="swapStrFixFloat" /></xsl:variable>
	<xsl:choose>
		<xsl:when test="not($ccy1 = $ccy2)">CrossCcy</xsl:when>
		<xsl:when test="($ff1 = 'Unknown') or ($ff2 = 'Unknown')">Unknown</xsl:when>
		<xsl:when test="$ff1 != $ff2">FixedFloat</xsl:when>
		<!-- FloatFloat or FixedFixed -->
		<xsl:otherwise><xsl:value-of select="concat($ff1,$ff2)" /></xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template mode="swapStrCcy" match="fpml:swapStream|fpml:capFloorStream"><xsl:value-of select="fpml:calculationPeriodAmount/fpml:calulation/fpml:notionalSchedule/fpml:notionalStepSchedule/fpml:currency"/></xsl:template>

<xsl:template mode="swapStrFixFloat" match="node()[fpml:calculationPeriodAmount/fpml:calculation/fpml:floatingRateCalculation]">Float</xsl:template>
<xsl:template mode="swapStrFixFloat" match="node()[fpml:calculationPeriodAmount/fpml:calculation/fpml:fixedRateSchedule]">Fixed</xsl:template>
<xsl:template mode="swapStrFixFloat" match="node()">Unknown</xsl:template>

<!-- swap uneven notional -->
<xsl:template mode="swapUneven" match="fpml:swap[fpml:swapStream/fpml:calculationPeriodAmount/fpml:calculation/fpml:notionalSchedule/fpml:notionalStepSchedule/fpml:step]">-Uneven</xsl:template>

<xsl:template mode="swapUneven" match="node()"/>

<!-- swap features -->
<xsl:template mode="swapFeatures" match="fpml:earlyTerminationProvision[fpml:mandatoryEarlyTermination]">-MandTerm</xsl:template>

<xsl:template mode="swapFeatures" match="fpml:earlyTerminationProvision[fpml:optionalEarlyTermination]">-OptTerm</xsl:template>

<xsl:template mode="swapFeatures" match="fpml:extendibleProvision">-Extendible</xsl:template>
<xsl:template mode="swapFeatures" match="fpml:cancelableProvision">-Cancelable</xsl:template>

<xsl:template mode="swapFeatures" match="node()"/>

<!-- swaption exercise type -->
<xsl:template mode="exerType" match="fpml:swaption[fpml:europeanExercise]">European</xsl:template>
<xsl:template mode="exerType" match="fpml:swaption[fpml:bermudaExercise]">Bermudan</xsl:template>
<xsl:template mode="exerType" match="fpml:swaption[fpml:americanExercise]">American</xsl:template>
<xsl:template mode="exerType" match="node()">Unknown</xsl:template>

<!-- CDS description -->
<xsl:template mode="productType" match="fpml:creditDefaultSwap">CDS</xsl:template>
<xsl:template mode="productType" match="dswg:creditDefaultSwapBasket">CDS-Basket</xsl:template>

<!-- FX description -->
<xsl:template mode="productType" match="fpml:fxSingleLeg">FXSpot/Forward<xsl:apply-templates mode="nonDeliv" select="."/></xsl:template>

<xsl:template mode="productType" match="fpml:fxSimpleOption">FXOption<xsl:apply-templates mode="nonDeliv" select="."/></xsl:template>

<xsl:template mode="productType" match="fpml:fxAverageRateOption">FXAverageRateOption<xsl:apply-templates mode="nonDeliv" select="."/></xsl:template>

<xsl:template mode="productType" match="fpml:fxBarrierOption">FXBarrierOption<xsl:apply-templates mode="nonDeliv" select="."/></xsl:template>

<xsl:template mode="productType" match="fpml:fxDigitalOption">FXDigitalOption<xsl:apply-templates mode="nonDeliv" select="."/></xsl:template>

<xsl:template mode="nonDeliv" match="fpml:fxSingleLeg[fpml:nonDeliverableForward]" >-NonDeliverable</xsl:template>
<xsl:template mode="nonDeliv" match="fpml:*[fpml:cashSettlementTerms]" >-NonDeliverable</xsl:template>

<xsl:template mode="nonDeliv" match="node()"/>

<!-- equity option description -->
<xsl:template mode="productType" match="fpml:equityOption">EquityOption-<xsl:apply-templates mode="eq-opt-type" select="fpml:equityExercise"/>-<xsl:value-of select="fpml:optionType"/>-<xsl:apply-templates mode="trsType" select="fpml:underlyer"/>-<xsl:apply-templates mode="trsNumber" select="fpml:underlyer"/><xsl:if test="fpml:strike/@xsi:type='dswg:AssetSwapStrike'">-AssetSwapStrike</xsl:if></xsl:template>

<!-- TRS description -->

<xsl:template mode="productType" match="dswg:varianceSwap">Vol/VarianceSwap-<xsl:apply-templates mode="trsType" select="fpml:varianceLeg"/>-<xsl:apply-templates mode="trsNumber" select="fpml:varianceLeg/fpml:underlyer"/><xsl:apply-templates mode="trsFeatures" select="."/></xsl:template>

<xsl:template mode="productType" match="fpml:equitySwap">TRS-<xsl:apply-templates mode="trsType" select="fpml:equityLeg"/>-<xsl:apply-templates mode="trsNumber" select="fpml:equityLeg/fpml:underlyer"/><xsl:apply-templates mode="trsFeatures" select="."/></xsl:template>

<xsl:template mode="trsType" match="fpml:equityLeg|fpml:varianceLeg"><xsl:apply-templates mode="trsType" select="fpml:underlyer"/></xsl:template>
<xsl:template mode="trsType" match="node()[fpml:singleUnderlyer]"><xsl:apply-templates select="fpml:singleUnderlyer" mode="undType"/></xsl:template>

<!-- determine underlyer type for a basket (the hard part is determining if it's a hybrid) -->
<xsl:template mode="trsType" match="node()[fpml:basket]">
	<!-- gen list of underlyers, in lexical order by fpml tag name -->
	<xsl:variable name="unds">
		<xsl:apply-templates select="fpml:basket/fpml:basketConstituent" mode="undElem">
			<xsl:sort select="local-name(*[1])" />
		</xsl:apply-templates>
	</xsl:variable>
	<!-- get first and last underlyers -->
	<xsl:variable name="first" select="common:node-set($unds)/*[1]"/>
	<xsl:variable name="last" select="common:node-set($unds)/*[last()]"/>
	<xsl:choose>
		<!-- if they're different, it's a hybrid, else it's the first (they're all the same) -->
		<xsl:when test="$first != $last">Hybrid</xsl:when>
		<xsl:otherwise><xsl:value-of select="$first"/></xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template mode="undElem" match="node()"><und><xsl:apply-templates select="." mode="undType"/></und></xsl:template>
<xsl:template mode="undType" match="node()[fpml:index]">ETIndex</xsl:template>
<xsl:template mode="undType" match="node()[fpml:equity]">Equity</xsl:template>
<xsl:template mode="undType" match="node()[fpml:bond]">Credit</xsl:template>
<xsl:template mode="undType" match="node()[fpml:convertibleBond]">ConvertibleBond</xsl:template>
<xsl:template mode="undType" match="node()[fpml:future]">Future</xsl:template>
<xsl:template mode="undType" match="node()[fpml:mutualFund]">MutualFund</xsl:template>

<xsl:template mode="trsNumber" match="node()[fpml:singleUnderlyer]">Single</xsl:template>
<xsl:template mode="trsNumber" match="node()[fpml:basket]">Basket</xsl:template>
<xsl:template mode="trsNumber" match="node()">Unknown</xsl:template>

<xsl:template mode="eq-opt-type" match="node()[fpml:equityAmericanExercise]">American</xsl:template>
<xsl:template mode="eq-opt-type" match="node()[fpml:equityEuropeanExercise]">European</xsl:template>
<xsl:template mode="eq-opt-type" match="node()[fpml:equityBermudaExercise]">European</xsl:template>
<xsl:template mode="eq-opt-type" match="node()">Unknown</xsl:template>
<!-- TRS features -->
<xsl:template mode="trsFeatures" match="*[fpml:earlyTermination]">-EarlyTerm</xsl:template>
<xsl:template mode="trsFeatures" match="fpml:equitySwap|dswg:varianceSwap" />

<xsl:template mode="productType" match="dswg:shortForm[fpml:productType]" priority="10"><xsl:value-of select="fpml:productType"/></xsl:template>
<xsl:template mode="productType" match="dswg:shortForm" priority="5">Generic</xsl:template>

<xsl:template mode="validate-productType" match="dswg:shortForm" priority="10">
	<xsl:variable name="provided" select="fpml:productType" />
	<xsl:variable name="front" select="substring-before($provided, '-')"/>
	<xsl:variable name="back" select="substring-after($provided, '-')"/>
	<xsl:if test="($front != 'IR') and ($front != 'FX') and ($front != 'EQ') and ($front != 'CD') and ($front != 'COM')">(DSWG.700) WARNING - supplied asset class (<xsl:value-of select="$front"/>) does not match expected value ([IR|EQ|CD|FX|COM]) in <xsl:apply-templates select="." mode="location"/></xsl:if>
	<xsl:if test="($back != 'Swap') and ($back != 'Cap') and ($back != 'Floor') and ($back != 'Option')">(DSWG.701) ERROR - supplied product type (<xsl:value-of select="$back"/>) does not match expected value ([Swap|Cap|Floor|Option]) in <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<!-- open questions
Equity Swaps
LegAmount - should we restrict the choice, e.g. to reference amount?  this area is a bit vague in the FpML

-->
	
<!-- skip references -->
<xsl:template mode="validate-missing" match="node()[@href]"/>

<!-- skip intentionally empty CDS fields -->
<xsl:template mode="validate-missing" match="fpml:creditEvents" priority="10"/>
<xsl:template mode="validate-missing" match="fpml:deliverableObligations" priority="10"/>

<xsl:template mode="validate-missing" match="node()[*]">
	<xsl:apply-templates mode="validate-missing" select="*"/> 
</xsl:template>

<xsl:template mode="validate-missing" match="node()[not(*)]" priority="-1">
	<xsl:if test="not(text())">
DSWG.111 ERROR - missing data content in element at <xsl:apply-templates select="." mode="location"/></xsl:if>
</xsl:template>

<!-- statistics -->

<xsl:variable name="prodsTF">
	<prod elem="swap" label="IRS"/>
	<prod elem="swaption" label="IR Swaption"/>
	<prod elem="creditDefaultSwap" label="CDS"/>
	<prod elem="creditDefaultSwapBasket" label="CDSBasket"/>
	<prod elem="equitySwap" label="TRS"/>
	<prod elem="equityOption" label="Equity/Asset Swap Option"/>
	<prod elem="varianceSwap" label="Variance Swap"/>
	<prod elem="fxSingleLeg" label="FX Fwd"/>
	<prod elem="fxSimpleOption" label="FX Simple Option"/>
	<prod elem="fxDigitalOption" label="FX Digital Option"/>
	<prod elem="fxBarrierOption" label="FX Barrier Option"/>
	<prod elem="fxAverageRateOption" label="FX Average Rate Option"/>
	<prod elem="shortForm" label="Short Form Product"/>
</xsl:variable>
<xsl:variable name="prods" select="common:node-set($prodsTF)" />

<xsl:template match = "fpml:FpML" mode="stats">
	<xsl:text>
=====================================
INFO: Statistics:</xsl:text>
Total positions: <xsl:value-of select="count(dswg:position)"/><xsl:text>
</xsl:text>
	<xsl:variable name="pos" select="dswg:position/dswg:positionSummary/*[2]"/>
	<!-- count known product tags -->
	<xsl:for-each select="$prods/*">
		<xsl:variable name="prodtag" select="@elem"/>
		<xsl:variable name="prodcount" select="count($pos[local-name(.)=$prodtag])"/>
		<xsl:if test="$prodcount &gt; 0">
			<xsl:value-of select="@label"/>: <xsl:value-of select="$prodcount"/><xsl:text>
</xsl:text>
		</xsl:if>
	</xsl:for-each>
	<!-- count unknown product tags -->
	<xsl:variable name="unknownsTF">
		<xsl:for-each select="$pos">
			<xsl:variable name="tag" select="local-name(.)"/>
			<xsl:if test="count($prods/*[@elem=$tag])=0"><uk><xsl:value-of select="$tag"/></uk></xsl:if>
		</xsl:for-each>
	</xsl:variable>
	<xsl:variable name="unknowns" select="common:node-set($unknownsTF)"/>
Unknown product tags: <xsl:value-of select="count($unknowns/*)"/>
	<xsl:if test="count($unknowns/*) &gt; 0">
		<xsl:text> (</xsl:text>
		<xsl:for-each select="$unknowns/*">
			<xsl:value-of select = "text()"/>
		</xsl:for-each>
		<xsl:text>)</xsl:text>
	</xsl:if>


</xsl:template>
	
</xsl:stylesheet>
